import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The hero of your game. This is a moving actor in the ScrollingWorld.
 * You control the movement of this actor with the arrows on the keyboard.
 * The hero cannot move through rocks.
 * When the hero goes further than thresholdX or thresholdY from the center of the world,
 * it makes the world scroll.
 * 
 * @author frederic.boulanger@centralesupelec.fr
 * @version 2015-03-10
 */
public class Hero extends MovingActor {
    private static final int deltaX = 5;
    private static final int deltaY = 5;
    private static final int thresholdX = 50;
    private static final int thresholdY = 50;
    /**
     * Move according to the arrow keys.
     */
    @Override
	public void act() {
        if (Greenfoot.isKeyDown("up")) {
            move(0, -deltaY);
        }
        if (Greenfoot.isKeyDown("down")) {
            move(0, deltaY);
        }
        if (Greenfoot.isKeyDown("left")) {
            move(-deltaX, 0);
        }
        if (Greenfoot.isKeyDown("right")) {
            move(deltaX, 0);
        }
    }
    
    /**
     * Perform the actual move, checking for stones and making the world scroll in
     * order to stay in a dead zone around the center of the world.
     */
    public void move(int dx, int dy) {
        ScrollingWorld myWorld = (ScrollingWorld)getWorld();
        int nx = getX() + dx;
        int ny = getY() + dy;
        if (myWorld.getObjectsAt(nx, ny, Rock.class).size() == 0) {
            setLocation(nx, ny);
            if (  Math.abs(nx - myWorld.getWidth()/2) > thresholdX
               || Math.abs(ny - myWorld.getHeight()/2) > thresholdY
               ) {
                   myWorld.scroll(dx, dy);
            }
        }
    }
}
