import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * This world distinguishes between fixed, or anchored actors, which do not move,
 * and moving actors, which may leave the visible window on the world.
 * The scroll method moves this window, making "anchored actors" stay in place in the window,
 * and "moving actors" move, possibly leaving the window and becoming invisible.
 * 
 * @author frederic.boulanger@centralesupelec.fr
 * @version 2015-04-10
 */
public class ScrollingWorld extends World {

    /**
     * Constructor for objects of class ScrollingWorld.
     * 
     */
    public ScrollingWorld() {    
        // Create a new world with 800x600 cells with a cell size of 1x1 pixels.
        super(800, 600, 1, false);
        // Build a rectangle of AnchoredStuff actors to frame the world
        GreenfootImage img = new AnchoredStuff().getImage();
        int stepy = img.getHeight();
        int stepx = img.getWidth();
        // Bottom and top rows
        for (int x = stepx; x < getWidth(); x += stepx) {
            addObject(new AnchoredStuff(), x, stepy);
            addObject(new AnchoredStuff(), x, getHeight() - stepy);
        }
        // Left and right columns
        for (int y = 2*stepy; y < getHeight() - 2*stepy; y += stepy) {
            addObject(new AnchoredStuff(), getWidth() - stepx, y);
            addObject(new AnchoredStuff(), stepx, y);
        }
        // Add 6 stones at some random places
        for (int i = 0; i < 6; i++) {
            addObject(new Rock(),
                      2*stepx + Greenfoot.getRandomNumber(getWidth() - 4*stepx),
                      2*stepy + Greenfoot.getRandomNumber(getHeight() - 4*stepy)
            );
        }
        // Add the hero (the actor which moves according to the keyboard arrows)
        addObject(new Hero(), getWidth()/2, getHeight()/2);
    }
    
    // Scroll all moving actors by dx, dy
    public void scroll(int dx, int dy) {
        for (Object o : this.getObjects(MovingActor.class)) {
            Actor a = (Actor)o;
            a.setLocation(a.getX() - dx, a.getY() - dy);
        }
    }

}
